/**********************************************************************************************/
/*                                                                                            */
/*                              .                                    */
/*                                                                                            */
/**********************************************************************************************/
#include  "asynctmr.h"


/*--------------------------------------------------------------------------------------------*/
/*    .                                                   */
/*--------------------------------------------------------------------------------------------*/
struct TIMER Async_Timer[8];

/*--------------------------------------------------------------------------------------------*/
/*    .                                              */
/* :     .                                                          */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
void Async_Timer_Init(void)
{
  //     .
  for(register IDX i = 0; i < 8; i++) Async_Timer[i].Flags = 0;
  //   0    .
  TIMSK_OCIE0 = TIMSK_TOIE0 = 0;    //     0.
  ASSR_AS0 = 1;                     //   .
  TCNT0 = OCR0 = 0;                 //    0  255.
  TCCR0 = 0x05;                     //  = 128 (32768 / 128 = 256).
  //      0.
  while((ASSR_TCN0UB != 0) && (ASSR_OCR0UB != 0) && (ASSR_TCR0UB != 0));
  //    .
  TIFR_OCF0 = TIFR_TOV0 = 1;
  TIMSK_TOIE0 = 1;
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   .                                                                 */
/* :     (0 - 7),                                                        */
/*                   (1..65535),                                        */
/*                   ( NULL),                           */
/*                .                                                   */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
__monitor void Start_Async_Timer(IN IDX timer, IN WORD interval , IN pTIMER_FUNC func, IN bool cycle)
{
  Async_Timer[timer].Interval = interval;
  Async_Timer[timer].Counter = interval;
  Async_Timer[timer].pFunc = func;
  Async_Timer[timer].Timer_Cycle = cycle;
  Async_Timer[timer].Timer_Ena = TIMER_ENA;
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*   .                                                               */
/* :     (0 - 7).                                                        */
/* :    .                                                         */
/*--------------------------------------------------------------------------------------------*/
__monitor void Stop_Async_Timer(IN IDX timer)
{
  Async_Timer[timer].Timer_Ena = TIMER_DIS;
  return;
}

/*--------------------------------------------------------------------------------------------*/
/*                                                                                            */
/*                           0.                      */
/*                                                                                            */
/*--------------------------------------------------------------------------------------------*/
#pragma vector = TIMER0_OVF_vect
__interrupt void Timer0_Ovf_Interrupt(void)
{
  //    .
  __enable_interrupt();
  //          .
  for(register IDX timer = 0; timer < 8; timer++)
     {
       if((Async_Timer[timer].Timer_Ena == TIMER_ENA) && (--Async_Timer[timer].Counter == 0))
         {
           //  . ,     .
           if(Async_Timer[timer].pFunc == NULL)
             {
               //       -   .
               Async_Timer[timer].Timer_Ena = TIMER_DIS;
             }
           else
             {
               //      .
               //     
               // (..        ).
               if(Async_Timer[timer].Timer_Cycle == TIMER_CYCLE) Async_Timer[timer].Counter = Async_Timer[timer].Interval;
               else Async_Timer[timer].Timer_Ena = TIMER_DIS;
               (*Async_Timer[timer].pFunc)();
             }
         }
     }
  return;
}


/**********************************************************************************************/
